@@ -1,4 +1,4 @@
-# Copyright 2018-2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright 2018-2021 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the 'License'). You
 # may not use this file except in compliance with the License. A copy of
@@ -31,10 +31,10 @@ logging.getLogger("paramiko").setLevel(logging.INFO)
 
 class WorkerRunner(process.ProcessRunner):
     """Runner responsible for preparing MPI distributed training and waiting for MPI
-     master execution to finish.
+    master execution to finish.
     """
 
-    def __init__(self, user_entry_point, args, env_vars, master_hostname):
+    def __init__(self, user_entry_point, args, env_vars, processes_per_host, master_hostname):
         """Initialize a WorkerRunner, which is responsible for preparing distributed
         training with MPI and waiting for MPI master execution to finish.
 
@@ -44,7 +44,7 @@ class WorkerRunner(process.ProcessRunner):
             env_vars (dict(str,str)): A dictionary of environment variables.
             master_hostname (str): The master hostname.
         """
-        super(WorkerRunner, self).__init__(user_entry_point, args, env_vars)
+        super(WorkerRunner, self).__init__(user_entry_point, args, env_vars, processes_per_host)
         self._master_hostname = str(master_hostname)
 
     def run(
@@ -116,9 +116,9 @@ class MasterRunner(process.ProcessRunner):
         user_entry_point,
         args,
         env_vars,
+        processes_per_host,
         master_hostname,
         hosts,
-        process_per_host,
         custom_mpi_options,
         network_interface_name,
         interval=1,
@@ -134,21 +134,20 @@ class MasterRunner(process.ProcessRunner):
             env_vars (dict(str,str)): A dictionary of environment variables.
             master_hostname (str): The master hostname.
             hosts ([str]): A list of hosts.
-            process_per_host (int): Number of processes per host.
+            processes_per_host (int): Number of processes per host.
             custom_mpi_options (str): A string of custom MPI options to be parsed.
             network_interface_name (str): The network interface name.
             interval (int or float): The interval at which to check the connection in seconds.
                 Defaults to 1 second.
             timeout_in_seconds (int): The number of seconds to wait for workers. Defaults to
                 3600 seconds (ie. 1 hour).
-            num_processes (int): The total number of processes.
         """
 
-        super(MasterRunner, self).__init__(user_entry_point, args, env_vars)
+        super(MasterRunner, self).__init__(user_entry_point, args, env_vars, processes_per_host)
 
         self._master_hostname = master_hostname
         self._hosts = hosts
-        self._process_per_host = process_per_host
+        self._processes_per_host = processes_per_host
         self._num_processes = num_processes
         self._custom_mpi_options = custom_mpi_options
         self._network_interface_name = network_interface_name
@@ -174,16 +173,16 @@ class MasterRunner(process.ProcessRunner):
 
     def _create_command(self):
         num_hosts = len(self._hosts)
-        num_processes = self._num_processes or self._process_per_host * num_hosts
+        num_processes = self._num_processes or self._processes_per_host * num_hosts
 
         # By default, use one process per GPU, or one process per node (if training with CPU).
-        if self._process_per_host == 1:
+        if self._processes_per_host == 1:
             host_list = self._hosts
         else:
-            host_list = ["%s:%s" % (host, self._process_per_host) for host in self._hosts]
+            host_list = ["%s:%s" % (host, self._processes_per_host) for host in self._hosts]
 
         msg = "Env Hosts: %s Hosts: %s process_per_hosts: %s num_processes: %s"
-        logger.info(msg, self._hosts, host_list, self._process_per_host, num_processes)
+        logger.info(msg, self._hosts, host_list, self._processes_per_host, num_processes)
 
         overridden_known_options, additional_options = _parse_custom_mpi_options(
             self._custom_mpi_options
@@ -249,7 +248,6 @@ class MasterRunner(process.ProcessRunner):
             command.extend(["-x", name])
 
         command.extend(super(MasterRunner, self)._create_command())
-
         return command
 
     def _python_command(self):
@@ -291,9 +289,9 @@ def _start_sshd_daemon():  # type: () -> None
 def _can_connect(host, port=22):  # type: (str, int) -> bool
     """Check if the connection to provided ``host`` and ``port`` is possible.
 
-       Args:
-           host (str): Hostname for the host to check connection.
-           port (int): Port name of the host to check connection on.
+    Args:
+        host (str): Hostname for the host to check connection.
+        port (int): Port name of the host to check connection on.
     """
     try:
         logger.debug("Testing connection to host %s", host)
